Package: simmr
Type: Package
Title: A Stable Isotope Mixing Model
Version: 0.5.2
Date: 2026-4-30
Authors@R: c(
    person("Emma", "Govan", role = "aut"),
    person("Andrew", "Parnell", email = "andrew.parnell1@ucd.ie",  role = c("cre", "aut"))
    )
URL: https://github.com/andrewcparnell/simmr,
        https://andrewcparnell.github.io/simmr/
BugReports: https://github.com/andrewcparnell/simmr/issues
Language: en-US
Description: Fits Stable Isotope Mixing Models (SIMMs) and is meant as a longer term replacement to the previous widely-used package SIAR. SIMMs are used to infer dietary proportions of organisms consuming various food sources from observations on the stable isotope values taken from the organisms' tissue samples. However SIMMs can also be used in other scenarios, such as in sediment mixing or the composition of fatty acids. The main functions are simmr_load() and simmr_mcmc(). The two vignettes contain a quick start and a full listing of all the features. The methods used are detailed in the papers Parnell et al 2010 <doi:10.1371/journal.pone.0009672>, and Parnell et al 2013 <doi:10.1002/env.2221>.
Depends: R (>= 3.5.0), R2jags, ggplot2
Imports: compositions, boot, reshape2, graphics, stats, viridis,
        bayesplot, checkmate, Rcpp, GGally
Encoding: UTF-8
License: GPL (>= 2)
LazyData: TRUE
Suggests: knitr, rmarkdown, readxl, testthat, covr, vdiffr, tibble,
        ggnewscale
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2026-05-02 06:17:32 UTC; andrewparnell
RoxygenNote: 7.3.3
Repository: CRAN
Date/Publication: 2026-05-11 08:40:02 UTC
LinkingTo: Rcpp, RcppArmadillo, RcppDist,
Author: Emma Govan [aut],
  Andrew Parnell [cre, aut]
Maintainer: Andrew Parnell <andrew.parnell1@ucd.ie>
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-11 11:25:07 UTC; unix
Archs: simmr.so.dSYM
