# Generated by roxygen2: do not edit by hand

S3method(combine_sources,simmr_output)
S3method(compare_groups,simmr_output)
S3method(compare_sources,simmr_output)
S3method(plot,simmr_input)
S3method(plot,simmr_output)
S3method(posterior_predictive,simmr_output)
S3method(print,simmr_input)
S3method(print,simmr_output)
S3method(prior_viz,simmr_output)
S3method(simmr_mcmc,simmr_input)
S3method(summary,simmr_output)
export(combine_sources)
export(compare_groups)
export(compare_sources)
export(posterior_predictive)
export(prior_viz)
export(simmr_elicit)
export(simmr_ffvb)
export(simmr_load)
export(simmr_mcmc)
import(checkmate)
import(ggplot2)
import(graphics)
import(viridis)
importFrom(R2jags,jags)
importFrom(Rcpp,sourceCpp)
importFrom(bayesplot,ppc_intervals)
importFrom(bayesplot,ppc_intervals_grouped)
importFrom(boot,logit)
importFrom(compositions,clrInv)
importFrom(reshape2,"melt")
importFrom(stats,"cor")
importFrom(stats,cor)
importFrom(stats,optim)
importFrom(stats,rnorm)
importFrom(stats,sd)
useDynLib(simmr, .registration = TRUE)
