% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.simmr_output.R
\name{print.simmr_output}
\alias{print.simmr_output}
\title{Print simmr output object}
\usage{
\method{print}{simmr_output}(x, ...)
}
\arguments{
\item{x}{An object of class \code{simmr_output}}

\item{...}{Other arguments (not supported)}
}
\value{
Returns a neat summary of the object
}
\description{
Print simmr output object
}
\seealso{
\code{\link{simmr_mcmc}} and \code{\link{simmr_ffvb}} for creating
\code{simmr_output} objects
}
